#include "newcontactdialog.h"
#include "schatcontactlist.h"

#include <qvariant.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qmime.h>
#include <qdragobject.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmessagebox.h>


static QPixmap uic_load_pixmap_NewContactDialog( const QString &name )
{
    const QMimeSource *m = QMimeSourceFactory::defaultFactory()->data( name );
    if ( !m )
	return QPixmap();
    QPixmap pix;
    QImageDrag::decode( m, pix );
    return pix;
}
/* 
 *  Constructs a NewContactDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
NewContactDialog::NewContactDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
       setName( "NewContactDialog" );
    setEnabled( TRUE );
    resize( 250, 133 ); 
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, sizePolicy().hasHeightForWidth() ) );
    setCaption( trUtf8( "New Contact" ) );

    text_label = new QLabel( this, "text_label" );
    text_label->setGeometry( QRect( 70, 10, 120, 30 ) );
    text_label->setText( trUtf8( "Enter IP address" ) );

    ip_edit_field = new QLineEdit( this, "ip_edit_field" );
    ip_edit_field->setGeometry( QRect( 20, 40, 211, 31 ) );
    ip_edit_field->setFrameShape( QLineEdit::LineEditPanel );
    ip_edit_field->setFrameShadow( QLineEdit::Sunken );


    button_cancel = new QPushButton( this, "button_cancel" );
    button_cancel->setGeometry( QRect( 150, 80, 81, 31 ) );
    button_cancel->setText( trUtf8( "Cancel" ) );


    button_ok = new QPushButton( this, "button_ok" );
    button_ok->setGeometry( QRect( 20, 80, 81, 31 ) );
    button_ok->setText( trUtf8( "OK" ) );

    // signals and slots connections
    connect( button_cancel, SIGNAL( released() ), this, SLOT( buttonCancelReleased() ) );
    connect( button_ok, SIGNAL( released() ), this, SLOT( buttonOkReleased() ) );
    connect( ip_edit_field, SIGNAL( returnPressed() ), this, SLOT( buttonOkReleased() ) );

    my_parent=( SchatContactList* )parent;

    init();
}

/*
 *  Destroys the object and frees any allocated resources
 */
NewContactDialog::~NewContactDialog()
{
    // no need to delete child widgets, Qt does it all for us
}


void NewContactDialog::init()
{
//
}



void NewContactDialog::buttonCancelReleased()
{
    this->hide();
}


void NewContactDialog::buttonOkReleased()
{
    QString address;
    QHostAddress *ipAddress = new QHostAddress();
    address = ip_edit_field->text();

    if( !ipAddress->setAddress( address ) )
    {
        QMessageBox mb( "Simple Chat",
        "Wrong IP address",
        QMessageBox::Information,
        QMessageBox::Ok,
        QMessageBox::NoButton,
        QMessageBox::NoButton );
        mb.exec();
        return;
    }

    my_parent->newIPAddress( ipAddress );
    this->hide();
}

