#include "newcontactdialog.h"
#include "schatcontactlist.h"
#include "currentchat.h"
#include "unixsocket.h"
#include "list.h"

#include <qvariant.h>
#include <qlistbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qmime.h>
#include <qdragobject.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtextedit.h>



/* 
 *  Constructs a SchatContactList which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 */
SchatContactList::SchatContactList( QWidget* parent,  const char* name, WFlags fl )
    : QMainWindow( parent, name, fl )
{
    if ( !name )
        setName( "SchatContactList" );
    resize( 576, 408 );
    setCaption( trUtf8( "Simple Chat" ) );
    setCentralWidget( new QWidget( this, "qt_central_widget" ) );

    tab_widget = new QTabWidget( centralWidget(), "tab_widget" );
    tab_widget->setGeometry( QRect( 180, 12, 391, 365 ) );


    contact_list = new QListBox( centralWidget(), "contact_list" );
    contact_list->setGeometry( QRect( 10, 0, 161, 371 ) );

    // actions

    new_connection_dialog_action = new QAction( this, "new_connection_dialog_action" );
    new_connection_dialog_action->setText( trUtf8( "New Connection" ) );
    new_connection_dialog_action->setMenuText( trUtf8( "&New Connection" ) );
    new_connection_dialog_action->setWhatsThis( trUtf8( "Create a new connection" ) );
    new_connection_dialog_action->setAccel( 4194382 );

    delete_connection_action = new QAction( this, "delete_connection_action" );
    delete_connection_action->setText( trUtf8( "Delete Connection" ) );
    delete_connection_action->setMenuText( trUtf8( "&Delete Connection" ) );
    delete_connection_action->setWhatsThis( trUtf8( "Detele current connection" ) );
    delete_connection_action->setAccel( 4194372 );

    opions_action = new QAction( this, "opions_action" );
    opions_action->setText( trUtf8( "Options" ) );
    opions_action->setMenuText( trUtf8( "&Options" ) );
    opions_action->setWhatsThis( trUtf8( "Options menu" ) );
    opions_action->setAccel( 4194383 );

    exit_action = new QAction( this, "exit_action" );
    exit_action->setText( trUtf8( "Exit" ) );
    exit_action->setMenuText( trUtf8( "E&xit" ) );
    exit_action->setAccel( 4194385 );


    // menu_bar
    menu_bar = new QMenuBar( this, "menu_bar" );

    popup_menu = new QPopupMenu( this );
    new_connection_dialog_action->addTo( popup_menu );
    delete_connection_action->addTo( popup_menu );
    opions_action->addTo( popup_menu );
    exit_action->addTo( popup_menu );
    menu_bar->insertItem( trUtf8( "System" ), popup_menu );

    connect( exit_action, SIGNAL( activated() ), this, SLOT( close() ) );
    connect( new_connection_dialog_action, SIGNAL( activated() ), this, SLOT( showDialog() ) );
    connect( contact_list, SIGNAL( doubleClicked(QListBoxItem*) ), this, SLOT( contactListDoubleClicked(QListBoxItem*) ) );
    connect( contact_list, SIGNAL( returnPressed(QListBoxItem*) ), this, SLOT( contactListDoubleClicked(QListBoxItem*) ) );
    connect( this, SIGNAL( messageReceived( QString*, QHostAddress*) ), this, SLOT( messageHandle( QString*, QHostAddress*) ) );
    init();
    QThread::start();
}

/*
 *  Destroys the object and frees any allocated resources
 */
SchatContactList::~SchatContactList()
{
    // no need to delete child widgets, Qt does it all for us
}


void SchatContactList::init()
{
    contact_dialog = new NewContactDialog( this );
    list = new List();
    unix_socket = new UnixSocket();
}

void SchatContactList::run()
{
    while( TRUE )
    {
        QString *message;
        QHostAddress *from;

        unix_socket->Receive( &message, &from );

        emit messageReceived( message, from );
    }
}

void SchatContactList::helpIndex()
{

}

void SchatContactList::helpContents()
{

}

void SchatContactList::helpAbout()
{

}

void SchatContactList::showDialog()
{
    contact_dialog->show();
}

/** This function adds IP address to contact list */
void SchatContactList::newIPAddress( QHostAddress *new_address )
{
    int i;
    int index;


    if( ( i = list->getIndex( new_address ) ) >= 0 )
    {
       contact_list->setSelected( list->getListBoxItem( i ), TRUE );
       return;
    }


    index = list->add( new_address );

    QString *alias = new QString( new_address->toString() );


    list->add( alias, index );

    contact_list->insertItem( *alias );
    contact_list->setSelected( contact_list->count() - 1, TRUE );

    list->add( contact_list->item( contact_list->count() - 1 ), index );


    contact_list->sort();
}

/** No descriptions */
void SchatContactList::contactListDoubleClicked( QListBoxItem *list_box_item )
{
    int i;


    i = list->getIndex( list_box_item );

    if( list->getCurrentChat( i ) != NULL )
    {
        int index = list->getTabIndex( i );

        tab_widget->setTabEnabled( tab_widget->page( index ) , TRUE );
        tab_widget->showPage( tab_widget->page( index ) );
        return;
    }

    current_chat = new CurrentChat( unix_socket, this, NULL, 0,
                    list->getIP( i ), list->getAlias( i ) );
    tab_widget->addTab( current_chat, list_box_item->text() );
    tab_widget->setTabEnabled( current_chat, TRUE );
    tab_widget->showPage( current_chat );


    list->add( current_chat, i );
    list->setTabIndex( tab_widget->count() - 1, i );

    current_chat->show();
}

/** slot, connected to messageReceived. This function handles received message */
void SchatContactList::messageHandle( QString *message, QHostAddress *from )
{
    int index;

    index = list->getIndex( from );

    if( index >= 0 )
    {
        current_chat = list->getCurrentChat( index );
        current_chat->chat->append( from->toString() + QString( " wrote:" ) );
        current_chat->chat->append( *message );
        // TODO bolding list item
        return;
    }

    newIPAddress( from );

    index = list->getIndex( from );

    current_chat = new CurrentChat( unix_socket, this, "Current Chat" );
    current_chat = new CurrentChat( unix_socket, this, NULL, 0,
                    list->getIP( index ), list->getAlias( index ) );
    tab_widget->addTab( current_chat, *list->getAlias( index ) );

    tab_widget->setTabEnabled( current_chat, TRUE );
    tab_widget->showPage( current_chat );

    list->add( current_chat, index );
    list->setTabIndex( tab_widget->count() - 1, index );

    current_chat->show();


    list->getCurrentChat( index )->chat->append( from->toString() + QString( " wrote:" ) );
    list->getCurrentChat( index )->chat->append( *message );
}
